==> smart-ssh-client-0.0.2-1.1-any.pkg.tar.zst: usr/lib/python3.12/site-packages/smart_ssh_client/proxy.py <==
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2023-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""SSH proxy specific functions."""

import array
import os
import socket
import sys
from typing import Optional

from .exceptions import SmartSSHClientError
from .portknock import port_knock
from .utils import resolve_host

__all__ = ["run_proxy"]


def run_proxy(  # noqa: CFQ004
    ssh_host: str, ssh_port: int, ssh_knock: Optional[str] = None, only_family: Optional[int] = None
) -> int:
    """Run in proxy mode."""

    # We only check our args if we're run in proxy mode
    if not ssh_host:
        print("PROXY ERROR: Smart SSH Client - No smart ssh hostname provided", file=sys.stderr)
        return 1
    if not ssh_port:
        print("PROXY ERROR: Smart SSH Client - No smart ssh port provided", file=sys.stderr)
        return 1

    print(
        f"PROXY INFO: Smart SSH Client - SSH host '{ssh_host}', port '{ssh_port}'",
        file=sys.stderr,
    )

    # Resolve hostname to an IP address
    try:
        target_family, target_ip = resolve_host(ssh_host, only_family)
    except SmartSSHClientError as e:
        print(f"PROXY ERROR: Smart SSH Client - {e}", file=sys.stderr)
        return 1

    # Perform port knocking if specified
    if ssh_knock:
        port_knock(target_family, target_ip, ssh_knock)

    # Establishing TCP connection
    try:
        with socket.socket(target_family, socket.SOCK_STREAM) as ssh_sock:
            print(
                f"PROXY INFO: Smart SSH Client - Connecting to '{target_ip}' on port '{ssh_port}'",
                file=sys.stderr,
            )
            ssh_sock.connect((target_ip, int(ssh_port)))

            print(
                f"PROXY INFO: Smart SSH Client - Connected to '{target_ip}' on port '{ssh_port}'",
                file=sys.stderr,
            )
            # Pass the file descriptor:
            fds = array.array("i", [ssh_sock.fileno()])
            sockdata = [(socket.SOL_SOCKET, socket.SCM_RIGHTS, fds)]
            socket.socket(fileno=1).sendmsg([b"\0"], sockdata)
            # NK: stdout cannot be used beyond this point
            sys.stdout = open(os.devnull, "w", encoding="UTF-8")  # noqa: SIM115 # pylint: disable=consider-using-with
            return 0

    except OSError as e:
        print(
            f"PROXY ERROR: Smart SSH Client - Failed connecting to '{target_ip}': {e}",
            file=sys.stderr,
        )

    return 1

==> smart-ssh-client-0.1.2-1.1-any.pkg.tar.zst: usr/lib/python3.12/site-packages/smart_ssh_client/proxy.py <==
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2023-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""SSH proxy specific functions."""

import array
import os
import pathlib
import socket
import sys

from .exceptions import SmartSSHClientError
from .portknock import port_knock
from .utils import resolve_host

__all__ = ["run_proxy"]


def run_proxy(
    ssh_host: str,
    ssh_port: int,
    ssh_knock: str | None = None,
    only_family: int | None = None,
) -> int:
    """Run in proxy mode."""

    # We only check our args if we're run in proxy mode
    if not ssh_host:
        print("PROXY ERROR: Smart SSH Client - No smart ssh hostname provided", file=sys.stderr)  # noqa: T201
        return 1
    if not ssh_port:
        print("PROXY ERROR: Smart SSH Client - No smart ssh port provided", file=sys.stderr)  # noqa: T201
        return 1

    print(  # noqa: T201
        f"PROXY INFO: Smart SSH Client - SSH host '{ssh_host}', port '{ssh_port}'",
        file=sys.stderr,
    )

    # Resolve hostname to an IP address
    try:
        target_family, target_ip = resolve_host(ssh_host, only_family)
    except SmartSSHClientError as e:
        print(f"PROXY ERROR: Smart SSH Client - {e}", file=sys.stderr)  # noqa: T201
        return 1

    # Perform port knocking if specified
    if ssh_knock:
        port_knock(target_family, target_ip, ssh_knock)

    # Establishing TCP connection
    try:
        with socket.socket(target_family, socket.SOCK_STREAM) as ssh_sock:
            print(  # noqa: T201
                f"PROXY INFO: Smart SSH Client - Connecting to '{target_ip}' on port '{ssh_port}'",
                file=sys.stderr,
            )
            ssh_sock.connect((target_ip, int(ssh_port)))

            print(  # noqa: T201
                f"PROXY INFO: Smart SSH Client - Connected to '{target_ip}' on port '{ssh_port}'",
                file=sys.stderr,
            )
            # Pass the file descriptor:
            fds = array.array("i", [ssh_sock.fileno()])
            sockdata = [(socket.SOL_SOCKET, socket.SCM_RIGHTS, fds)]
            socket.socket(fileno=1).sendmsg([b"\0"], sockdata)
            # NK: stdout cannot be used beyond this point
            sys.stdout = pathlib.Path(os.devnull).open("w", encoding="UTF-8")  # noqa: SIM115
            return 0

    except OSError as e:
        print(  # noqa: T201
            f"PROXY ERROR: Smart SSH Client - Failed connecting to '{target_ip}': {e}",
            file=sys.stderr,
        )

    return 1
